@@ -10,13 +10,15 @@
 #include "dot.h"
 #include "layout.h"
 #include "ftextstream.h"
+#include "config.h"
+#include "docparser.h"
 
 //----------------------------------------------------------------------
 // method implementation
 
 static int g_dirCount=0;
 
-DirDef::DirDef(const char *path) : Definition(path,1,path)
+DirDef::DirDef(const char *path) : Definition(path,1,1,path), visited(FALSE)
 {
   bool fullPathNames = Config_getBool("FULL_PATH_NAMES");
   // get display name (stipping the paths mentioned in STRIP_FROM_PATH)
@@ -34,7 +36,7 @@ DirDef::DirDef(const char *path) : Definition(path,1,path)
   }
   setLocalName(m_shortName);
   m_dispName = fullPathNames ? stripFromPath(path) : m_shortName;
-  if (m_dispName.at(m_dispName.length()-1)=='/')
+  if (m_dispName.length()>0 && m_dispName.at(m_dispName.length()-1)=='/')
   { // strip trailing /
     m_dispName = m_dispName.left(m_dispName.length()-1);
   }
@@ -136,7 +138,7 @@ void DirDef::writeDetailedDescription(OutputList &ol,const QCString &title)
     // repeat brief description
     if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF"))
     {
-      ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE);
+      ol.generateDoc(briefFile(),briefLine(),this,0,briefDescription(),FALSE,FALSE);
     }
     // separator between brief and details
     if (!briefDescription().isEmpty() && Config_getBool("REPEAT_BRIEF") && 
@@ -148,6 +150,7 @@ void DirDef::writeDetailedDescription(OutputList &ol,const QCString &title)
         // ol.newParagraph();  // FIXME:PARA
         ol.enableAll();
         ol.disableAllBut(OutputGenerator::Man);
+        ol.enable(OutputGenerator::Latex);
         ol.writeString("\n\n");
       ol.popGeneratorState();
     }
@@ -155,7 +158,7 @@ void DirDef::writeDetailedDescription(OutputList &ol,const QCString &title)
     // write documentation
     if (!documentation().isEmpty())
     {
-      ol.parseDoc(docFile(),docLine(),this,0,documentation()+"\n",TRUE,FALSE);
+      ol.generateDoc(docFile(),docLine(),this,0,documentation()+"\n",TRUE,FALSE);
     }
   }
 }
@@ -164,29 +167,31 @@ void DirDef::writeBriefDescription(OutputList &ol)
 {
   if (!briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
   {
-    ol.startParagraph();
-    ol.parseDoc(briefFile(),briefLine(),this,0,briefDescription(),TRUE,FALSE);
-    ol.pushGeneratorState();
-    ol.disable(OutputGenerator::RTF);
-    ol.writeString(" \n");
-    ol.enable(OutputGenerator::RTF);
-
-    if (Config_getBool("REPEAT_BRIEF") ||
-        !documentation().isEmpty()
-       )
+    DocRoot *rootNode = validatingParseDoc(
+         briefFile(),briefLine(),this,0,briefDescription(),TRUE,FALSE);
+    if (rootNode && !rootNode->isEmpty())
     {
-      ol.disableAllBut(OutputGenerator::Html);
-      ol.startTextLink(0,"details");
-      ol.parseText(theTranslator->trMore());
-      ol.endTextLink();
-    }
-    ol.popGeneratorState();
+      ol.startParagraph();
+      ol.writeDoc(rootNode,this,0);
+      ol.pushGeneratorState();
+      ol.disable(OutputGenerator::RTF);
+      ol.writeString(" \n");
+      ol.enable(OutputGenerator::RTF);
 
-    //ol.pushGeneratorState();
-    //ol.disable(OutputGenerator::RTF);
-    //ol.newParagraph();
-    //ol.popGeneratorState();
-    ol.endParagraph();
+      if (Config_getBool("REPEAT_BRIEF") ||
+          !documentation().isEmpty()
+         )
+      {
+        ol.disableAllBut(OutputGenerator::Html);
+        ol.startTextLink(0,"details");
+        ol.parseText(theTranslator->trMore());
+        ol.endTextLink();
+      }
+      ol.popGeneratorState();
+
+      ol.endParagraph();
+    }
+    delete rootNode;
   }
   ol.writeSynopsis();
 }
@@ -220,8 +225,9 @@ void DirDef::writeSubDirList(OutputList &ol)
     ol.parseText(theTranslator->trDir(TRUE,FALSE));
     ol.endMemberHeader();
     ol.startMemberList();
-    DirDef *dd=m_subdirs.first();
-    while (dd)
+    QListIterator<DirDef> it(m_subdirs);
+    DirDef *dd;
+    for (;(dd=it.current());++it)
     {
       ol.startMemberDeclaration();
       ol.startMemberItem(dd->getOutputFileBase(),0);
@@ -229,14 +235,10 @@ void DirDef::writeSubDirList(OutputList &ol)
       ol.insertMemberAlign();
       ol.writeObjectLink(dd->getReference(),dd->getOutputFileBase(),0,dd->shortName());
       ol.endMemberItem();
-      if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
-      {
-        Doxygen::tagFile << "    <dir>" << convertToXML(dd->displayName()) << "</dir>" << endl;
-      }
       if (!dd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
         ol.startMemberDescription(dd->getOutputFileBase());
-        ol.parseDoc(briefFile(),briefLine(),dd,0,dd->briefDescription(),
+        ol.generateDoc(briefFile(),briefLine(),dd,0,dd->briefDescription(),
             FALSE, // indexWords
             FALSE, // isExample
             0,     // exampleName
@@ -246,7 +248,6 @@ void DirDef::writeSubDirList(OutputList &ol)
         ol.endMemberDescription();
       }
       ol.endMemberDeclaration(0,0);
-      dd=m_subdirs.next();
     }
 
     ol.endMemberList();
@@ -262,8 +263,9 @@ void DirDef::writeFileList(OutputList &ol)
     ol.parseText(theTranslator->trFile(TRUE,FALSE));
     ol.endMemberHeader();
     ol.startMemberList();
-    FileDef *fd=m_fileList->first();
-    while (fd)
+    QListIterator<FileDef> it(*m_fileList);
+    FileDef *fd;
+    for (;(fd=it.current());++it)
     {
       ol.startMemberDeclaration();
       ol.startMemberItem(fd->getOutputFileBase(),0);
@@ -291,15 +293,11 @@ void DirDef::writeFileList(OutputList &ol)
         ol.endTextLink();
         ol.popGeneratorState();
       }
-      if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
-      {
-        Doxygen::tagFile << "    <file>" << convertToXML(fd->name()) << "</file>" << endl;
-      }
       ol.endMemberItem();
       if (!fd->briefDescription().isEmpty() && Config_getBool("BRIEF_MEMBER_DESC"))
       {
         ol.startMemberDescription(fd->getOutputFileBase());
-        ol.parseDoc(briefFile(),briefLine(),fd,0,fd->briefDescription(),
+        ol.generateDoc(briefFile(),briefLine(),fd,0,fd->briefDescription(),
             FALSE, // indexWords
             FALSE, // isExample
             0,     // exampleName
@@ -309,7 +307,6 @@ void DirDef::writeFileList(OutputList &ol)
         ol.endMemberDescription();
       }
       ol.endMemberDeclaration(0,0);
-      fd=m_fileList->next();
     }
     ol.endMemberList();
   }
@@ -325,12 +322,69 @@ void DirDef::endMemberDeclarations(OutputList &ol)
   ol.endMemberSections();
 }
 
+QCString DirDef::shortTitle() const
+{
+  return theTranslator->trDirReference(m_shortName);
+}
+
+bool DirDef::hasDetailedDescription() const
+{
+  static bool repeatBrief = Config_getBool("REPEAT_BRIEF");
+  return (!briefDescription().isEmpty() && repeatBrief) || !documentation().isEmpty();
+}
+
+void DirDef::writeTagFile(FTextStream &tagFile)
+{
+  tagFile << "  <compound kind=\"dir\">" << endl;
+  tagFile << "    <name>" << convertToXML(displayName()) << "</name>" << endl;
+  tagFile << "    <path>" << convertToXML(name()) << "</path>" << endl;
+  tagFile << "    <filename>" << convertToXML(getOutputFileBase()) << Doxygen::htmlFileExtension << "</filename>" << endl;
+  QListIterator<LayoutDocEntry> eli(
+      LayoutDocManager::instance().docEntries(LayoutDocManager::Directory));
+  LayoutDocEntry *lde;
+  for (eli.toFirst();(lde=eli.current());++eli)
+  {
+    switch (lde->kind())
+    {
+      case LayoutDocEntry::DirSubDirs:
+        {
+          if (m_subdirs.count()>0)
+          {
+            DirDef *dd;
+            QListIterator<DirDef> it(m_subdirs);
+            for (;(dd=it.current());++it)
+            {
+              tagFile << "    <dir>" << convertToXML(dd->displayName()) << "</dir>" << endl;
+            }
+          }
+        }
+        break;
+      case LayoutDocEntry::DirFiles:
+        {
+          if (m_fileList->count()>0)
+          {
+            QListIterator<FileDef> it(*m_fileList);
+            FileDef *fd;
+            for (;(fd=it.current());++it)
+            {
+              tagFile << "    <file>" << convertToXML(fd->name()) << "</file>" << endl;
+            }
+          }
+        }
+        break;
+      default:
+        break;
+    }
+  }
+  writeDocAnchorsToTagFile(tagFile);
+  tagFile << "  </compound>" << endl;
+}
+
 void DirDef::writeDocumentation(OutputList &ol)
 {
   static bool generateTreeView = Config_getBool("GENERATE_TREEVIEW");
   ol.pushGeneratorState();
   
-  QCString shortTitle=theTranslator->trDirReference(m_shortName);
   QCString title=theTranslator->trDirReference(m_dispName);
   startFile(ol,getOutputFileBase(),name(),title,HLI_None,!generateTreeView);
 
@@ -344,7 +398,7 @@ void DirDef::writeDocumentation(OutputList &ol)
   startTitle(ol,getOutputFileBase());
   ol.pushGeneratorState();
     ol.disableAllBut(OutputGenerator::Html);
-    ol.parseText(shortTitle);
+    ol.parseText(shortTitle());
     ol.enableAll();
     ol.disable(OutputGenerator::Html);
     ol.parseText(title);
@@ -352,14 +406,6 @@ void DirDef::writeDocumentation(OutputList &ol)
   endTitle(ol,getOutputFileBase(),title);
   ol.startContents();
 
-  if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
-  {
-    Doxygen::tagFile << "  <compound kind=\"dir\">" << endl;
-    Doxygen::tagFile << "    <name>" << convertToXML(displayName()) << "</name>" << endl;
-    Doxygen::tagFile << "    <path>" << convertToXML(name()) << "</path>" << endl;
-    Doxygen::tagFile << "    <filename>" << convertToXML(getOutputFileBase()) << Doxygen::htmlFileExtension << "</filename>" << endl;
-  }
-  
   //---------------------------------------- start flexible part -------------------------------
 
   SrcLangExt lang = getLanguage();
@@ -402,10 +448,12 @@ void DirDef::writeDocumentation(OutputList &ol)
       case LayoutDocEntry::ClassAllMembersLink:
       case LayoutDocEntry::ClassUsedFiles:
       case LayoutDocEntry::NamespaceNestedNamespaces:
+      case LayoutDocEntry::NamespaceNestedConstantGroups:
       case LayoutDocEntry::NamespaceClasses:
       case LayoutDocEntry::NamespaceInlineClasses:
       case LayoutDocEntry::FileClasses:
       case LayoutDocEntry::FileNamespaces:
+      case LayoutDocEntry::FileConstantGroups:
       case LayoutDocEntry::FileIncludes:
       case LayoutDocEntry::FileIncludeGraph:
       case LayoutDocEntry::FileIncludedByGraph: 
@@ -433,12 +481,6 @@ void DirDef::writeDocumentation(OutputList &ol)
 
   //---------------------------------------- end flexible part -------------------------------
 
-  if (!Config_getString("GENERATE_TAGFILE").isEmpty()) 
-  {
-    writeDocAnchorsToTagFile();
-    Doxygen::tagFile << "  </compound>" << endl;
-  }
-
   ol.endContents();
 
   endFileWithNavPath(this,ol);
@@ -574,12 +616,10 @@ bool DirDef::depGraphIsTrivial() const
 
 //----------------------------------------------------------------------
 
-int FilePairDict::compareItems(GCI item1,GCI item2)
+int FilePairDict::compareValues(const FilePair *left,const FilePair *right) const
 {
-  FilePair *left  = (FilePair*)item1;
-  FilePair *right = (FilePair*)item2;
-  int orderHi = stricmp(left->source()->name(),right->source()->name());
-  int orderLo = stricmp(left->destination()->name(),right->destination()->name());
+  int orderHi = qstricmp(left->source()->name(),right->source()->name());
+  int orderLo = qstricmp(left->destination()->name(),right->destination()->name());
   return orderHi==0 ? orderLo : orderHi;
 }
 
@@ -628,7 +668,7 @@ bool DirDef::matchPath(const QCString &path,QStrList &l)
   while (s)
   {
     QCString prefix = s;
-    if (stricmp(prefix.left(path.length()),path)==0) // case insensitive compare
+    if (qstricmp(prefix.left(path.length()),path)==0) // case insensitive compare
     {
       return TRUE;
     }
@@ -648,7 +688,7 @@ DirDef *DirDef::mergeDirectoryInTree(const QCString &path)
   while ((i=path.find('/',p))!=-1)
   {
     QCString part=path.left(i+1);
-    if (!matchPath(part,Config_getList("STRIP_FROM_PATH")) && part!="/")
+    if (!matchPath(part,Config_getList("STRIP_FROM_PATH")) && (part!="/" && part!="//"))
     {
       dir=createNewDir(part); 
     }
@@ -784,7 +824,7 @@ static void computeCommonDirPrefix()
           QCString dirName = dir->name();
           if (dirName.length()>path.length())
           {
-            if (strncmp(dirName,path,l)!=0) // dirName does not start with path
+            if (qstrncmp(dirName,path,l)!=0) // dirName does not start with path
             {
               int i=path.findRev('/',l-2);
               if (i==-1) // no unique prefix -> stop
