@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: groupdef.h,v 1.18 2001/03/19 19:27:40 root Exp $
+ * 
  *
- * Copyright (C) 1997-2012 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,13 +18,10 @@
 #ifndef GROUPDEF_H
 #define GROUPDEF_H
 
-#include "qtbc.h"
 #include "sortdict.h"
 #include "definition.h"
-#include "memberlist.h"
-#include "memberdef.h"
-#include "htmlhelp.h"
 
+class MemberList;
 class FileList;
 class ClassSDict;
 class FileDef;
@@ -41,6 +38,8 @@ class DirDef;
 class DirList;
 class FTVHelp;
 class Entry;
+class MemberDef;
+class FTextStream;
 
 /** A model of a group of symbols. */
 class GroupDef : public Definition
@@ -69,6 +68,7 @@ class GroupDef : public Definition
     void writeDocumentation(OutputList &ol);
     void writeMemberPages(OutputList &ol);
     void writeQuickMemberLinks(OutputList &ol,MemberDef *currentMd) const;
+    void writeTagFile(FTextStream &);
     int  countMembers() const;
     bool isLinkableInProject() const;
     bool isLinkable() const;
@@ -81,6 +81,7 @@ class GroupDef : public Definition
 
     void addListReferences();
     void sortMemberLists();
+    bool subGrouping() const { return m_subGrouping; }
 
     bool visited;    // number of times accessed for output - KPW
 
@@ -90,7 +91,7 @@ class GroupDef : public Definition
     void setGroupScope(Definition *d) { groupScope = d; }
     Definition *getGroupScope() const { return groupScope; }
 
-    MemberList *getMemberList(MemberList::ListType lt) const;
+    MemberList *getMemberList(MemberListType lt) const;
     const QList<MemberList> &getMemberLists() const { return m_memberLists; }
 
     /* user defined member groups */
@@ -103,6 +104,7 @@ class GroupDef : public Definition
     PageSDict *     getPages() const        { return pageDict; }
     DirList *       getDirs() const         { return dirList; }
     PageSDict *     getExamples() const     { return exampleDict; }
+    bool hasDetailedDescription() const;
     //MemberList*     getMembers() const      { return allMemberList; }
     void sortSubGroups();
     
@@ -110,11 +112,11 @@ class GroupDef : public Definition
     void addMemberListToGroup(MemberList *,bool (MemberDef::*)() const);
 
   private: 
-    MemberList *createMemberList(MemberList::ListType lt);
-    void addMemberToList(MemberList::ListType lt,MemberDef *md);
-    void writeMemberDeclarations(OutputList &ol,MemberList::ListType lt,const QCString &title);
-    void writeMemberDocumentation(OutputList &ol,MemberList::ListType lt,const QCString &title);
-    void removeMemberFromList(MemberList::ListType lt,MemberDef *md);
+    MemberList *createMemberList(MemberListType lt);
+    void addMemberToList(MemberListType lt,MemberDef *md);
+    void writeMemberDeclarations(OutputList &ol,MemberListType lt,const QCString &title);
+    void writeMemberDocumentation(OutputList &ol,MemberListType lt,const QCString &title);
+    void removeMemberFromList(MemberListType lt,MemberDef *md);
     void writeGroupGraph(OutputList &ol);
     void writeFiles(OutputList &ol,const QCString &title);
     void writeNamespaces(OutputList &ol,const QCString &title);
@@ -152,6 +154,7 @@ class GroupDef : public Definition
 
     QList<MemberList> m_memberLists;
     MemberGroupSDict *memberGroupSDict;
+    bool              m_subGrouping;
 
 };
 
@@ -161,9 +164,10 @@ class GroupSDict : public SDict<GroupDef>
   public:
     GroupSDict(uint size) : SDict<GroupDef>(size) {}
     virtual ~GroupSDict() {}
-    int compareItems(GCI item1,GCI item2)
+  private:
+    int compareValues(const GroupDef *item1,const GroupDef *item2) const
     {
-      return strcmp(((GroupDef*)item1)->groupTitle(),((GroupDef*)item2)->groupTitle());
+      return qstrcmp(item1->groupTitle(),item2->groupTitle());
     }
 };
 
@@ -171,9 +175,9 @@ class GroupSDict : public SDict<GroupDef>
 class GroupList : public QList<GroupDef>
 {
   public:
-    int compareItems(GCI item1,GCI item2)
+    int compareValues(const GroupDef *item1,const GroupDef *item2) const
     {
-      return strcmp(((GroupDef*)item1)->groupTitle(),((GroupDef*)item2)->groupTitle());
+      return qstrcmp(item1->groupTitle(),item2->groupTitle());
     }
 };
 
