@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: membergroup.cpp,v 1.19 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2010 by Dimitri van Heesch.
+ * Copyright (C) 1997-2014 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -15,7 +15,6 @@
  *
  */
 
-#include "qtbc.h"
 #include "membergroup.h"
 #include "memberlist.h"
 #include "outputlist.h"
@@ -46,7 +45,7 @@ MemberGroup::MemberGroup(Definition *parent,
       int id,const char *hdr,const char *d,const char *docFile) 
 {
   //printf("New member group id=%d header=%s desc=%s\n",id,hdr,d);
-  memberList      = new MemberList(MemberList::memberGroup);
+  memberList      = new MemberList(MemberListType_memberGroup);
   grpId           = id;
   grpHeader       = hdr;
   doc             = d;
@@ -69,12 +68,14 @@ MemberGroup::~MemberGroup()
 void MemberGroup::insertMember(MemberDef *md)
 {
   //printf("MemberGroup::insertMember m_parent=%s memberList=%p count=%d"
-  //       " member section list: %p\n",
+  //       " member section list: %p: md=%p:%s\n",
   //       m_parent ? m_parent->name().data() : "<null>",
   //       memberList->first() ? memberList->first()->getSectionList(m_parent) : 0,
   //       memberList->count(),
-  //       md->getSectionList(m_parent));
-  MemberDef *firstMd = memberList->first();
+  //       md->getSectionList(m_parent),
+  //       md,md->name().data());
+
+  MemberDef *firstMd = memberList->getFirst();
   if (inSameSection && memberList->count()>0 && 
       firstMd->getSectionList(m_parent)!=md->getSectionList(m_parent))
   {
@@ -99,32 +100,34 @@ void MemberGroup::insertMember(MemberDef *md)
 }
 
 
-void MemberGroup::setAnchors(ClassDef *context)
+void MemberGroup::setAnchors()
 {
-  ::setAnchors(context,'z',memberList,grpId);
+  ::setAnchors(memberList);
 }
 
 void MemberGroup::writeDeclarations(OutputList &ol,
-               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd)
+               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
+               bool showInline)
 {
-  //printf("MemberGroup::writeDeclarations() memberList->count()=%d\n",memberList->count());
+  //printf("MemberGroup::writeDeclarations() %s\n",grpHeader.data());
   QCString ldoc = doc;
-  if (!ldoc.isEmpty()) ldoc.prepend("<a name=\""+anchor()+"\"></a>");
-  memberList->writeDeclarations(ol,cd,nd,fd,gd,grpHeader,ldoc/*,TRUE,TRUE*/);
+  if (!ldoc.isEmpty()) ldoc.prepend("<a name=\""+anchor()+"\" id=\""+anchor()+"\"></a>");
+  memberList->writeDeclarations(ol,cd,nd,fd,gd,grpHeader,ldoc,FALSE,showInline);
 }
 
 void MemberGroup::writePlainDeclarations(OutputList &ol,
-               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd
+               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
+               ClassDef *inheritedFrom,const char *inheritId
               )
 {
   //printf("MemberGroup::writePlainDeclarations() memberList->count()=%d\n",memberList->count());
-  memberList->writePlainDeclarations(ol,cd,nd,fd,gd);
+  memberList->writePlainDeclarations(ol,cd,nd,fd,gd,inheritedFrom,inheritId);
 }
 
 void MemberGroup::writeDocumentation(OutputList &ol,const char *scopeName,
-               Definition *container)
+               Definition *container,bool showEnumValues,bool showInline)
 {
-  memberList->writeDocumentation(ol,scopeName,container,0);
+  memberList->writeDocumentation(ol,scopeName,container,0,showEnumValues,showInline);
 }
 
 void MemberGroup::writeDocumentationPage(OutputList &ol,const char *scopeName,
@@ -133,6 +136,43 @@ void MemberGroup::writeDocumentationPage(OutputList &ol,const char *scopeName,
   memberList->writeDocumentationPage(ol,scopeName,container);
 }
 
+void MemberGroup::addGroupedInheritedMembers(OutputList &ol,ClassDef *cd,
+               MemberListType lt,
+               ClassDef *inheritedFrom,const QCString &inheritId)
+{
+  //printf("** addGroupedInheritedMembers()\n");
+  MemberListIterator li(*memberList);
+  MemberDef *md;
+  for (li.toFirst();(md=li.current());++li)
+  {
+    //printf("matching %d == %d\n",lt,md->getSectionList(m_parent)->listType());
+    if (lt==md->getSectionList(m_parent)->listType())
+    {
+      MemberList ml(lt);
+      ml.append(md);
+      ml.writePlainDeclarations(ol,cd,0,0,0,inheritedFrom,inheritId);
+    }
+  }
+}
+
+int MemberGroup::countGroupedInheritedMembers(MemberListType lt)
+{
+  //printf("** countGroupedInheritedMembers()\n");
+  int count=0;
+  MemberListIterator li(*memberList);
+  MemberDef *md;
+  for (li.toFirst();(md=li.current());++li)
+  {
+    //printf("matching %d == %d\n",lt,md->getSectionList(m_parent)->listType());
+    if (lt==md->getSectionList(m_parent)->listType())
+    {
+      count++;
+    }
+  }
+  return count;
+}
+
+
 /*! Add this group as a subsection of the declaration section, instead
  *  of rendering it in its own section
  */
@@ -146,11 +186,11 @@ void MemberGroup::addToDeclarationSection()
   }
 }
 
-int MemberGroup::countDecMembers(/*bool sectionPerType*/)
+int MemberGroup::countDecMembers(GroupDef *gd)
 {
   if (m_numDecMembers==-1) /* number of member not cached */
   {
-    memberList->countDecMembers(/*TRUE,TRUE,sectionPerType*/);
+    memberList->countDecMembers(gd);
     m_numDecMembers = memberList->numDecMembers();
   }
   return m_numDecMembers;
@@ -166,11 +206,18 @@ int MemberGroup::countDocMembers()
   return m_numDocMembers;
 }
 
+int MemberGroup::countInheritableMembers(ClassDef *inheritedFrom) const
+{
+  return memberList->countInheritableMembers(inheritedFrom);
+}
+
+
 void MemberGroup::distributeMemberGroupDocumentation()
 {
   //printf("MemberGroup::distributeMemberGroupDocumentation() %s\n",grpHeader.data());
-  MemberDef *md=memberList->first();
-  while (md)
+  MemberListIterator li(*memberList);
+  MemberDef *md;
+  for (li.toFirst();(md=li.current());++li)
   {
     //printf("checking md=%s\n",md->name().data());
     // find the first member of the group with documentation
@@ -182,16 +229,15 @@ void MemberGroup::distributeMemberGroupDocumentation()
       //printf("found it!\n");
       break;
     }
-    md=memberList->next();
   }
   if (md) // distribute docs of md to other members of the list
   {
     //printf("Member %s has documentation!\n",md->name().data());
-    MemberDef *omd=memberList->first();
-    while (omd)
+    MemberDef *omd;
+    for (li.toFirst();(omd=li.current());++li)
     {
-      if (md!=omd && omd->documentation().isEmpty() && 
-                     omd->briefDescription().isEmpty() && 
+      if (md!=omd && omd->documentation().isEmpty() &&
+                     omd->briefDescription().isEmpty() &&
                      omd->inbodyDocumentation().isEmpty()
          )
       {
@@ -200,7 +246,6 @@ void MemberGroup::distributeMemberGroupDocumentation()
         omd->setDocumentation(md->documentation(),md->docFile(),md->docLine());
         omd->setInbodyDocumentation(md->inbodyDocumentation(),md->inbodyFile(),md->inbodyLine());
       }
-      omd=memberList->next();
     }
   }
 }
@@ -281,7 +326,7 @@ void MemberGroup::addListReferences(Definition *def)
         name,
         theTranslator->trGroup(TRUE,TRUE),
         name,
-        grpHeader,0);
+        grpHeader,0,def);
   }
 }
 
@@ -343,6 +388,12 @@ void MemberGroup::setRefItems(const QList<ListItemInfo> *sli)
     } 
   }
 }
+
+void MemberGroup::writeTagFile(FTextStream &tagFile)
+{
+  memberList->writeTagFile(tagFile);
+}
+
 //--------------------------------------------------------------------------
 
 void MemberGroupInfo::setRefItems(const QList<ListItemInfo> *sli)
